@@ -6,6 +6,7 @@ function( data,
              Y,
          ref.c,
          ref.p,
+          dist = c("poisson","binomial"),
          model = c("ns","bs","ls","factor"),
        dr.extr = c("weighted","Holford"),
           parm = c("ACP","APC","AdCP","AdPC","Ad-P-C","Ad-C-P","AC-P","AP-C"),
@@ -55,8 +56,15 @@ else
 # So presently we have variable A, P, D and Y in the local workspace.
 
 # Define reference period and cohort if not supplied
-p0 <- ifelse( missing( ref.p ), median( rep( P  , D ) ), ref.p )
-c0 <- ifelse( missing( ref.c ), median( rep( P-A, D ) ), ref.c )
+# Time-saving utility courtesy of Mike Murphy
+med <- function(x,y)
+{
+a <- y
+names(a) <- x
+return(as.numeric(names(a[cumsum(a)/sum(a)>.5][1])))
+}
+p0 <- ifelse( missing( ref.p ), med( P  , D), ref.p )
+c0 <- ifelse( missing( ref.c ), med( P-A, D), ref.c )
 
 # Redefine to logicals for whether a reference was requested or not
 ref.p <- !missing( ref.p )
@@ -147,8 +155,8 @@ else
     MP <- model.matrix( ~ factor( P ) - 1 )
     MC <- model.matrix( ~ factor( P-A ) - 1 )
     # The reference rows if required later
-    Rp <- MP[ abs( P   - p0 ) == min( abs( P   - p0 ) ),][1,]
-    Rc <- MC[ abs( P-A - c0 ) == min( abs( P-A - c0 ) ),][1,]
+    Rp <- MP[ abs( P   - p0 ) == min( abs( P   - p0 ) ),,drop=FALSE][1,]
+    Rc <- MC[ abs( P-A - c0 ) == min( abs( P-A - c0 ) ),,drop=FALSE][1,]
     }
   # End of factor model
 
@@ -159,12 +167,12 @@ else
     # Are knots supplied in a list?
     knl <- is.list( npar )
     if( knl ) nk <- sapply( npar, length )
-    MA <- if( knl ) ns( A  , knots=npar["A"][-c(1,nk[1])], Bo=npar["A"][c(1,nk[1])] )
-               else ns( A  ,    df=npar["A"] )
-    MP <- if( knl ) ns( P  , knots=npar["P"][-c(1,nk[2])], Bo=npar["P"][c(1,nk[2])] )
-               else ns( P  ,    df=npar["P"] )
-    MC <- if( knl ) ns( P-A, knots=npar["C"][-c(1,nk[3])], Bo=npar["C"][c(1,nk[3])] )
-               else ns( P-A,    df=npar["C"] )
+    MA <- if( knl ) ns( A  , knots=npar[["A"]][-c(1,nk[1])], Bo=npar[["A"]][c(1,nk[1])] )
+               else ns( A  ,    df=npar[["A"]] )
+    MP <- if( knl ) ns( P  , knots=npar[["P"]][-c(1,nk[2])], Bo=npar[["P"]][c(1,nk[2])] )
+               else ns( P  ,    df=npar[["P"]] )
+    MC <- if( knl ) ns( P-A, knots=npar[["C"]][-c(1,nk[3])], Bo=npar[["C"]][c(1,nk[3])] )
+               else ns( P-A,    df=npar[["C"]] )
     # The reference rows if required later
     Rp <- ns( p0, knots=attr( MP, "knots"), Boundary.knots=attr( MP, "Boundary.knots" ) )
     Rc <- ns( c0, knots=attr( MC, "knots"), Boundary.knots=attr( MC, "Boundary.knots" ) )
@@ -211,13 +219,27 @@ else
 # End of defining the three chunks of the model matrix that makes up the
 # model specification.
 
+# Fit the base model
+if( tolower(substr(dist,1,2)) == "po" )
+    {
+    m.APC <- glm( D ~ MA + I(P-p0) + MP + MC,
+                  offset =log( Y ), family = poisson )
+    Dist <- "Poisson with log(Y) offset"
+    }
+if( tolower(substr(dist,1,3)) %in% c("bin") )
+    {
+    m.APC <- glm( cbind( D, Y-D ) ~ MA + I(P-p0) + MP + MC,
+                  family = binomial )
+    Dist <- "Binomial regression (logistic) of D/Y"
+    }
+
 # Comparison of the 5 classical models:
 #--------------------------------------
-m.APC <- glm( D ~ MA + I(P-p0) + MP + MC + offset( log( Y ) ), family = poisson )
 m.AP  <- update( m.APC, . ~ . - MC )
 m.AC  <- update( m.APC, . ~ . - MP )
 m.Ad  <- update( m.AP , . ~ . - MP )
 m.A   <- update( m.Ad , . ~ . - I(P-p0) )
+m.0   <- update( m.A  , . ~ . - MA )
 AOV   <- anova( m.A, m.Ad, m.AC, m.APC, m.AP, m.Ad, test="Chisq" )
 # Change the header and row-names to readable form
 attr( AOV, "heading") <- "\nAnalysis of deviance for Age-Period-Cohort model\n"
@@ -282,8 +304,7 @@ if( length( grep( "-", parm ) ) == 0 )
   
   # We must fit a model with explicit drift to get the drift estimate
   if( parm %in% c("ADPC","ADCP","APC","ACP") )
-    m.APC <- glm( D ~ - 1 + MA + I(P-p0) + MPr + MCr + offset( log( Y ) ),
-                  family = poisson )
+    m.APC <- update( m.0, .~.-1 + MA + I(P-p0) + MPr + MCr )
                   
   # Extract the drift estimates
   drift <- rbind( ci.lin( m.APC, subset="I\\(", Exp=TRUE, alpha=alpha )[,5:7],
@@ -292,19 +313,16 @@ if( length( grep( "-", parm ) ) == 0 )
 
   # Then we can fit the other models if needed
   if( parm == "ADCP" )
-    m.APC <- glm( D ~ -1 + MA + I(P-A-c0) + MPr + MCr + offset( log( Y ) ),
-                  family = poisson )
+    m.APC <- update( m.0, .~.-1 + MA + I(P-A-c0) + MPr + MCr )
   if( parm == "APC" )
     {
     MPr <- cbind( P  -p0, MPr )
-    m.APC <- glm( D ~ -1 + MA + MPr + MCr + offset( log( Y ) ),
-                  family = poisson )
+    m.APC <- update( m.0, .~.-1 + MA + MPr + MCr )
     }
   if( parm == "ACP" )
     {
     MCr <- cbind( P-A-c0, MCr )
-    m.APC <- glm( D ~ -1 + MA + MPr + MCr + offset( log( Y ) ),
-                  family = poisson )
+    m.APC <- update( m.0, .~.-1 + MA + MPr + MCr )
     }
 
   # Then extract the effects:
@@ -321,17 +339,15 @@ if( length( grep( "-", parm ) ) == 0 )
   colnames( Per )[-1] <- c("P-RR",lu)
   colnames( Coh )[-1] <- c("C-RR",lu)
 
-  Type <- paste( "ML of APC-model (", parm, "):\n" )
+  Type <- paste( "ML of APC-model", Dist, ": (", parm, "):\n" )
 }
 # End of ML-options
 else
 { # Sequential approach (if a hyphen IS in the name)
 
   # Age-drift model always needed
-  adc <- glm( D ~ -1 + MA + I(P-A-c0) + offset( log( Y ) ),
-                       family=poisson )
-  adp <- glm( D ~ -1 + MA + I(P  -p0) + offset( log( Y ) ),
-                       family=poisson )
+  adc <- update( m.0, .~.-1 + MA + I(P-A-c0) )
+  adp <- update( m.0, .~.-1 + MA + I(P  -p0) )
   # The raw drift parameter
   drift <- ci.lin( adc, subset="I\\(", Exp=TRUE )[,5:7,drop=F]
   rownames( drift ) <- "A-d"
@@ -346,10 +362,8 @@ else
   if( parm == "AD-C-P" )
     {
     # Fit the two residual models and extract the parameters
-    rc <- glm( D ~ -1 + xC + offset( predict( adc, type="link" ) ),
-               family=poisson )
-    rp <- glm( D ~ -1 + xP + offset( predict(  rc, type="link" ) ),
-               family=poisson )
+    rc <- update( m.0, .~.-1 + xC, offset = predict( adc, type="link" ) )
+    rp <- update( m.0, .~.-1 + xP, offset = predict( adc, type="link" ) )
     A.eff <- ci.lin( adc, subset="MA", ctr.mat=MA[A.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     C.eff <- ci.lin(  rc, subset="xC", ctr.mat=xC[C.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     P.eff <- ci.lin(  rp, subset="xP", ctr.mat=xP[P.pos,], Exp=TRUE, alpha=alpha )[,5:7]
@@ -359,10 +373,8 @@ else
   if( parm == "AD-P-C" )
     {
     # Fit the two residual models in other sequence and extract the parameters
-    rp <- glm( D ~ -1 + xP + offset( predict( adp, type="link" ) ),
-               family=poisson )
-    rc <- glm( D ~ -1 + xC + offset( predict(  rp, type="link" ) ),
-               family=poisson )
+    rp <- update( m.0, .~.-1 + xP, offset = predict( adp, type="link" ) )
+    rc <- update( m.0, .~.-1 + xC, offset = predict(  rp, type="link" ) )
     A.eff <- ci.lin( adp, subset="MA", ctr.mat=MA[A.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     P.eff <- ci.lin(  rp, subset="xP", ctr.mat=xP[P.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     C.eff <- ci.lin(  rc, subset="xC", ctr.mat=xC[C.pos,], Exp=TRUE, alpha=alpha )[,5:7]
@@ -371,10 +383,8 @@ else
   else
   if( parm == "AC-P" )
     {
-    ac <- glm( D ~ -1 + MA + lC + offset( log( Y ) ),
-               family=poisson )
-    rp <- glm( D ~ -1 + xP + offset( predict( ac, type="link" ) ),
-               family=poisson )
+    ac <- update( m.0, .~.-1 + MA + lC )
+    rp <- update( m.0, .~.-1 + xP )
     A.eff <- ci.lin( ac, subset="MA", ctr.mat=MA[A.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     C.eff <- ci.lin( ac, subset="lC", ctr.mat=lC[C.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     P.eff <- ci.lin( rp, subset="xP", ctr.mat=xP[P.pos,], Exp=TRUE, alpha=alpha )[,5:7]
@@ -383,10 +393,8 @@ else
   else
   if( parm == "AP-C" )
     {
-    ap <- glm( D ~ -1 + MA + lP + offset( log( Y ) ),
-               family=poisson )
-    rc <- glm( D ~ -1 + xC + offset( predict( ap, type="link" ) ),
-               family=poisson )
+    ap <- update( m.0, .~.-1 + MA + lP )
+    rc <- update( m.0, .~.-1 + xC, offset = predict( ap, type="link" ) )
     A.eff <- ci.lin( ap, subset="MA", ctr.mat=MA[A.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     P.eff <- ci.lin( ap, subset="lP", ctr.mat=lP[P.pos,], Exp=TRUE, alpha=alpha )[,5:7]
     C.eff <- ci.lin( rc, subset="xC", ctr.mat=xC[C.pos,], Exp=TRUE, alpha=alpha )[,5:7]
@@ -401,7 +409,7 @@ else
   colnames( Per )[-1] <- c("P.eff",lu)
   colnames( Coh )[-1] <- c("C.eff",lu)
 
-  Type <- paste( "Sequential modelling (", parm, "):\n" )
+  Type <- paste( "Sequential modelling", Dist, ": (", parm, "):\n" )
 }
 # end of sequential approach
 
@@ -413,7 +421,8 @@ res <- list( Type=Type,
              Anova=AOV )
 if( model %in% c("ns","bs") ) res <- c( res, list( Knots=Knots) )
 res$Age[,-1] <- res$Age[,-1] * scale
-if( print.AOV ) print( res$Anova )
+if( print.AOV ) { print( res$Type )
+                  print( res$Anova ) }
 class( res ) <- "apc"
 invisible( res )
 }
